# -*- coding:utf-8 -*-

# Q's Web Diary
# Copyright (c) 2001,2011 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# ブログ
class BlogsController < ApplicationController

  before_filter :blog_must_be_owned_by_ug, :only => [:edit, :update, :destroy]

  before_filter :login_required, :only => [:login]


  # 公開ページのログインボタン
  def login
    @blog = Blog.find params[:blog_id]
    redirect_to "/blogs/#{@blog.id}/"
  end


  # ブログの一覧
  def index
    @blogs = Blog.all
  end


  # リード文、最新の記事などを表示
  def show
    @blog = Blog.find params[:id]
    @entries = Entry.paginate :page => params[:page], :per_page => 5,
                 :conditions => ["blog_id = ? AND status <> 0 AND date <= ?", 
                                 @blog.id, Date.today],
                 :order => "date DESC, id DESC"

    render :layout => "tdiary"
  end


  # 1ヶ月分 (一般に公開)
  def month
    @blog = Blog.find params[:blog_id]
    @mon = Date.new params[:year].to_i, params[:mon].to_i, 1
    @entries = Entry.find :all,
                    :conditions => ["blog_id = ? AND status <> 0 AND (date BETWEEN ? AND ?) AND date <= ?", @blog.id, @mon, (@mon >> 1) - 1, Date.today],
                    :order => "date, id"

    if @entries.empty?
      render :action => "not_found", :status => :not_found
      return
    else
      render :layout => "tdiary"
    end
  end


=begin
  # GET /diaries/new
  # GET /diaries/new.xml
  def new
    @diary = Diary.new

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @diary }
    end
  end
=end


  # GET /diaries/1/edit
  def edit
    @diary = Diary.find(params[:id])
  end


=begin
  # POST /diaries
  # POST /diaries.xml
  def create
    @diary = Diary.new(params[:diary])

    respond_to do |format|
      if @diary.save
        format.html { redirect_to(@diary, :notice => 'Diary was successfully created.') }
        format.xml  { render :xml => @diary, :status => :created, :location => @diary }
      else
        format.html { render :action => "new" }
        format.xml  { render :xml => @diary.errors, :status => :unprocessable_entity }
      end
    end
  end
=end


  # PUT /diaries/1
  # PUT /diaries/1.xml
  def update
    @diary = Diary.find(params[:id])

    respond_to do |format|
      if @diary.update_attributes(params[:diary])
        format.html { redirect_to(@diary, :notice => 'Diary was successfully updated.') }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @diary.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /diaries/1
  # DELETE /diaries/1.xml
  def destroy
    @diary = Diary.find(params[:id])
    @diary.destroy

    respond_to do |format|
      format.html { redirect_to(diaries_url) }
      format.xml  { head :ok }
    end
  end
end
