# -*- coding:utf-8 -*-

# Q's Web Diary
# Copyright (c) 2001,2011 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 一つ一つのエントリ
class EntriesController < ApplicationController

  # :login_required を兼ねる
  before_filter :blog_must_be_owned_by_ug, 
                :only => [ :new, :create, :edit, :update, :destroy, :preview,
                           :set_private, :set_public ]


=begin
  # GET /articles
  # GET /articles.xml
  def index
    @articles = Article.all

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @articles }
    end
  end
=end


  # 一般に公開
  def show
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:id]

    if @entry.status == 0 || @entry.date > Date.today
      if !current_user
        redirect_login
        return
      else
        render :action => "/forbidden", :status => :forbidden
        return
      end
    end

    render :layout => "tdiary"
  end


  def preview
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:entry_id]
  end


  # 新しいエントリ
  def new
    @blog = Blog.find params[:blog_id]
    @entry = Entry.new :blog => @blog

    @about = About.new
    @tags = ""
  end


  def norm_uri param
    if param.to_s.strip != ""
      uri = URI.parse normalize_kc(param).strip
      uri.normalize!
      if uri.fragment.to_s != "" && uri.fragment[0..0] != "!" # "#!"のときはママ
        uri.fragment = ""
      end
      return uri
    else
      return nil
    end
  end
  private :norm_uri


  def update_abouts! uri
    @about = About.find :first, 
                   :conditions => [ "uri_type = ? AND uri = ?", 1, uri.to_s ]
    if !@about
      @about = About.new
      @about.uri_type = 1 # URL
      @about.uri = uri.to_s
      @about.save!
    end
    ea = EntriesAbout.new :entry_id => @entry.id, :about_id => @about.id
    ea.save!
  end
  private :update_abouts!


  def update_tags! param
    @tags = param.to_s.strip
    normalize_kc(@tags).split(/[ \t]+/).each do |tname|
      tag = Tag.find_by_name tname.gsub("_", " ")
      if !tag
        tag = Tag.new :name => tname.gsub("_", " "),
                      :created_by => current_user
        tag.save!
      end
      et = EntriesTag.new :entry_id => @entry.id, :tag_id => tag.id
      et.save!
    end
  end
  private :update_tags!


  # 新しいエントリ作成 (実行)
  def create
    @blog = Blog.find params[:blog_id]
    @entry = Entry.new params[:entry]
    @entry.blog = @blog
    @entry.type = "Entry"
    @entry.status = 0 # draft
    @entry.create_user_id = current_user.id

    begin
      uri = norm_uri params[:about]
    rescue URI::InvalidURIError
      @about = About.new :uri => params[:about]
      @entry.errors.add_to_base "'〜について'が妥当なURIではありません。"
      @tags = params[:tags]
      render :action => "new"
      return
    end

    begin
      Entry.transaction do 
        @entry.save!
        update_abouts! uri if uri
        update_tags! params[:tags]
      end # transaction
    rescue ActiveRecord::RecordInvalid
      render :action => "new"
      return
    end

    if params[:draft]
      flash[:notice] = "新しいエントリを作成しました。"
      redirect_to "/blogs/#{@blog.id}/admin"
    else
      flash[:confirm] = true
      redirect_to "/blogs/#{@blog.id}/entries/#{@entry.id}/preview"
    end
  end


  # GET /articles/1/edit
  def edit
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:id]

    @tags = EntriesTag.find(:all, 
                :conditions => ["entry_id = ?", @entry.id]).collect do |et|
      et.tag.name.gsub(/[ \t]/, "_")
    end.join(" ")

    ea = EntriesAbout.find :first, :conditions => ["entry_id = ?", @entry.id] 
    @about = ea ? ea.about : About.new

    if params[:from] == "preview"
      @back_path = nil
    elsif params[:from] == "show"
      @back_path = "/blogs/#{@blog.id}/entries/#{@entry.id}"
    else
      @back_path = "/blogs/#{@blog.id}/admin"
    end
  end


  # PUT /articles/1
  # PUT /articles/1.xml
  def update
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:id]
    ea = EntriesAbout.find_by_entry_id @entry.id
    @about = ea ? ea.about : About.new

    @entry.attributes = params[:entry]
    @entry.status = 0 # draft

    begin
      uri = norm_uri params[:about]
    rescue URI::InvalidURIError
      @about.uri = params[:about]
      @entry.errors.add_to_base "'〜について'が妥当なURIではありません。"
      @tags = params[:tags]
      render :action => "edit"
      return
    end

    begin
      Entry.transaction do 
        # clean upのときに遅延評価させない
        old_abouts = @entry.abouts.collect do |x| x end
        old_tags = @entry.tags.collect do |x| x end

        # 一度消して作り直す
        EntriesAbout.delete_all ["entry_id = ?", @entry.id]
        EntriesTag.delete_all ["entry_id = ?", @entry.id]

        @entry.save!
        update_abouts! uri if uri
        update_tags! params[:tags] 

        # clean up
        old_abouts.each do |a| 
          a.destroy if !EntriesAbout.find_by_about_id(a.id)
        end
        old_tags.each do |t|
          t.destroy if !EntriesTag.find_by_tag_id(t.id)
        end
      end
    rescue ActiveRecord::RecordInvalid
      render :action => "edit"
      return
    end

    if params[:draft]
      flash[:notice] = 'Article was successfully updated.'
      redirect_to "/blogs/#{@blog.id}/admin"
    else
      flash[:confirm] = true
      redirect_to "/blogs/#{@blog.id}/entries/#{@entry.id}/preview"
    end
  end


  # DELETE /articles/1
  # DELETE /articles/1.xml
  def destroy
    @article = Article.find(params[:id])
    @article.destroy

    respond_to do |format|
      format.html { redirect_to(articles_url) }
      format.xml  { head :ok }
    end
  end

  
  def set_private
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:entry_id]

    @entry.status = 0
    @entry.save!
    flash[:notice] = "エントリをドラフト(非公開)化しました。"
    redirect_to "/blogs/#{@blog.id}/admin"
  end


  def set_public
    @blog = Blog.find params[:blog_id]
    @entry = Entry.find params[:entry_id]

    @entry.status = 1
    @entry.save!
    flash[:notice] = "エントリを公開しました。"
    redirect_to "/blogs/#{@blog.id}/entries/#{@entry.id}"
  end

end
